# Troubleshooting

## Verifying the effective device specification received by the device agent

When viewing a device resource using the command

```console
flightctl get device/${device_name} -o yaml
```

the output contains the device specification as specified by the user or the fleet controller based on the fleet's device template. That specification may contain references to configuration or secrets stored on external systems, such a Git or a Kubernetes cluster.

Only when the device agent queries the service, the service replaces these references with the actual configuration and secret data. While this better protects potentially sensitive data, it also makes troubleshooting faulty configurations hard.

Users with the `GetRenderedDevice` permission can run the following command to view the effective configuration as rendered by the service to the device agent:

```console
flightctl get device/${device_name} -o yaml --rendered
```

## Generating and downloading an SOS report

SOS reports are a collection of system information that can be used to debug issues with the device agent. The SOS report is generated by the `sos report` command, which is included in the device image.
To generate an SOS report, run the following console command on the device:

```console
flightctl console device/${device_name} -- sos report --batch --quiet
```

The output is a tarball named `sosreport-$hostname-$timestamp-$random-sequence.tar.xz`, whereby `$hostname` is the device's hostname and `$timestamp` is the current date and time. The tarball contains system information, logs, and configuration files that can be used to debug issues with the device agent.

Output example:

```console
sos report (version 4.8.1)


Your sos report has been generated and saved in:
       /var/tmp/sosreport-localhost-2025-04-28-svjuich.tar.xz

 Size  11.83MiB
 Owner root
 sha256        918563a260c5d6a069e178e9ddce643b3b93ad2a4fdca9a3e431b63b2e82041d

Please send this file to your support representative.
```

To download the SOS report, run the following command on the device:

```console
flightctl console device/${device_name} -- cat /var/tmp/sosreport-localhost-2025-04-28-svjuich.tar.xz > sosreport.tar.xz 
```
